/*
 *  handle.c
 * 
 *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
 *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
 *  Copyright (c) 2005, 2006 by Miklos Vajna <vmiklos@frugalware.org>
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */

#include "config.h"
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <sys/types.h>
#include <stdarg.h>
#include <syslog.h>
#include <libintl.h>
#include <time.h>
/* pacman */
#include "util.h"
#include "log.h"
#include "list.h"
#include "error.h"
#include "trans.h"
#include "alpm.h"
#include "server.h"
#include "handle.h"

pmhandle_t *_alpm_handle_new()
{
	pmhandle_t *handle;

	handle = (pmhandle_t *)malloc(sizeof(pmhandle_t));
	if(handle == NULL) {
		_alpm_log(PM_LOG_ERROR, _("malloc failure: could not allocate %d bytes"), sizeof(pmhandle_t));
		RET_ERR(PM_ERR_MEMORY, NULL);
	}

	memset(handle, 0, sizeof(pmhandle_t));
	handle->lckfd = -1;

#ifndef CYGWIN
	/* see if we're root or not */
	handle->uid = geteuid();
#ifndef FAKEROOT
	if(!handle->uid && getenv("FAKEROOTKEY")) {
		/* fakeroot doesn't count, we're non-root */
		handle->uid = 99;
	}
#endif

	/* see if we're root or not (fakeroot does not count) */
#ifndef FAKEROOT
	if(handle->uid == 0 && !getenv("FAKEROOTKEY")) {
#else
	if(handle->uid == 0) {
#endif
		handle->access = PM_ACCESS_RW;
	} else {
		handle->access = PM_ACCESS_RO;
	}
#else
	handle->access = PM_ACCESS_RW;
#endif

	handle->dbpath = strdup(PM_DBPATH);
	handle->cachedir = strdup(PM_CACHEDIR);

	return(handle);
}

int _alpm_handle_free(pmhandle_t *handle)
{
	ASSERT(handle != NULL, RET_ERR(PM_ERR_HANDLE_NULL, -1));

	/* close logfiles */
	if(handle->logfd) {
		fclose(handle->logfd);
		handle->logfd = NULL;
	}
	if(handle->usesyslog) {
		handle->usesyslog = 0;
		closelog();
	}

	/* free memory */
	FREETRANS(handle->trans);
	FREE(handle->root);
	FREE(handle->dbpath);
	FREE(handle->cachedir);
	FREE(handle->logfile);
	FREE(handle->xfercommand);
	FREELIST(handle->dbs_sync);
	FREELIST(handle->noupgrade);
	FREELIST(handle->noextract);
	FREELIST(handle->ignorepkg);
	FREELIST(handle->holdpkg);
	FREELIST(handle->needles);
	FREE(handle);

	return(0);
}

alpm_cb_log alpm_option_get_logcb() { return handle->logcb; }
alpm_cb_download alpm_option_get_dlcb() { return handle->dlcb; }
unsigned char alpm_option_get_logmask() { return handle->logmask; }
const char *alpm_option_get_root() { return handle->root; }
const char *alpm_option_get_dbpath() { return handle->dbpath; }
const char *alpm_option_get_cachedir() { return handle->cachedir; }
const char *alpm_option_get_logfile() { return handle->logfile; }
unsigned char alpm_option_get_usesyslog() { return handle->usesyslog; }
pmlist_t *alpm_option_get_noupgrades() { return handle->noupgrade; }
pmlist_t *alpm_option_get_noextracts() { return handle->noextract; }
pmlist_t *alpm_option_get_ignorepkgs() { return handle->ignorepkg; }
pmlist_t *alpm_option_get_holdpkgs() { return handle->holdpkg; }
time_t alpm_option_get_upgradedelay() { return handle->upgradedelay; }
const char *alpm_option_get_xfercommand() { return handle->xfercommand; }
unsigned short alpm_option_get_nopassiveftp() { return handle->nopassiveftp; }
unsigned short alpm_option_get_chomp() { return handle->chomp; }
pmlist_t *alpm_option_get_needles() { return handle->needles; }
unsigned short alpm_option_get_usecolor() { return handle->use_color; }

pmdb_t *alpm_option_get_localdb(pmhandle_t *handle) { return handle->db_local; }
pmlist_t *alpm_option_get_syncdbs(pmhandle_t *handle) { return handle->dbs_sync; }

void alpm_option_set_logcb(alpm_cb_log cb) { handle->logcb = cb; }

void alpm_option_set_dlcb(alpm_cb_download cb) { handle->dlcb = cb; }

void alpm_option_set_logmask(unsigned char mask) { handle->logmask = mask; }

void alpm_option_set_root(const char *root)
{
	if(handle->root) FREE(handle->root);
	if(root) handle->root = strdup(root);
}

void alpm_option_set_dbpath(const char *dbpath)
{
  if(handle->dbpath) FREE(handle->dbpath);
  if(dbpath) handle->dbpath = strdup(dbpath);
}

void alpm_option_set_cachedir(const char *cachedir)
{
  if(handle->cachedir) FREE(handle->cachedir);
  if(cachedir) handle->cachedir = strdup(cachedir);
}

void alpm_option_set_logfile(const char *logfile)
{
	if(handle->logfile) {
		FREE(handle->logfile);
		if(handle->logfd) {
			fclose(handle->logfd);
			handle->logfd = NULL;
		}
	}
	if(logfile) {
		handle->logfile = strdup(logfile);
		handle->logfd = fopen(logfile, "a");
	}
}

void alpm_option_set_usesyslog(unsigned char usesyslog) { handle->usesyslog = usesyslog; }

void alpm_option_add_noupgrade(char *pkg)
{
  handle->noupgrade = _alpm_list_add(handle->noupgrade, strdup(pkg));
}
void alpm_option_set_noupgrades(pmlist_t *noupgrade)
{
	if(handle->noupgrade) FREELIST(handle->noupgrade);
	if(noupgrade) handle->noupgrade = noupgrade;
}

void alpm_option_add_noextract(char *pkg)
{
  handle->noextract = _alpm_list_add(handle->noextract, strdup(pkg));
}
void alpm_option_set_noextracts(pmlist_t *noextract)
{
	if(handle->noextract) FREELIST(handle->noextract);
	if(noextract) handle->noextract = noextract;
}

void alpm_option_add_ignorepkg(char *pkg)
{
  handle->ignorepkg = _alpm_list_add(handle->ignorepkg, strdup(pkg));
}
void alpm_option_set_ignorepkgs(pmlist_t *ignorepkgs)
{
	if(handle->ignorepkg) FREELIST(handle->ignorepkg);
	if(ignorepkgs) handle->ignorepkg = ignorepkgs;
}

void alpm_option_add_holdpkg(char *pkg)
{
  handle->holdpkg = _alpm_list_add(handle->holdpkg, strdup(pkg));
}
void alpm_option_set_holdpkgs(pmlist_t *holdpkgs)
{
	if(handle->holdpkg) FREELIST(handle->holdpkg);
	if(holdpkgs) handle->holdpkg = holdpkgs;
}

void alpm_option_set_upgradedelay(time_t delay) { handle->upgradedelay = delay; }

void alpm_option_set_xfercommand(const char *cmd)
{
	if(handle->xfercommand) FREE(handle->xfercommand);
	if(cmd) handle->xfercommand = strdup(cmd);
}

void alpm_option_set_nopassiveftp(unsigned short nopasv) { handle->nopassiveftp = nopasv; }

void alpm_option_set_chomp(unsigned short chomp) { handle->chomp = chomp; }

void alpm_option_add_needle(char *needle)
{
  handle->needles = _alpm_list_add(handle->needles, strdup(needle));
}
void alpm_option_set_needles(pmlist_t *needles)
{
	if(handle->needles) FREELIST(handle->needles);
	if(needles) handle->needles = needles;
}
void alpm_option_set_usecolor(unsigned short usecolor) { handle->use_color = usecolor; }

/* vim: set ts=2 sw=2 et: */
